package middleware

import (
	"fmt"
	"net/http"

	"github.com/opentracing/opentracing-go"
	"github.com/opentracing/opentracing-go/ext"
	"hello-api/internal/app/parsers/devicecontext"
	"hello-api/internal/app/parsers/deviceresponseheader"
	"hello-api/internal/logger"
	"hello-api/internal/utils"

	"github.com/gin-gonic/gin"
)

func DeviceDeregistrationMiddleware(context *gin.Context) {
	deviceAuthContext, err := utils.ContextGet[*devicecontext.DeviceContext](context, DeviceContextHeader)
	if err != nil {
		_ = context.AbortWithError(http.StatusInternalServerError, DeviceContextNotFoundError)
		return
	}
	if deviceAuthContext.IsDeregistered() {
		handleDeregisteredDevice(context, deviceAuthContext)
	}
	context.Next()
}

func handleDeregisteredDevice(context *gin.Context, deviceAuthContext *devicecontext.DeviceContext) {
	orgKey := deviceAuthContext.GetOrgKey()
	logger.BaseLogger().Info(fmt.Sprintf("Deregistered device %d in org %s", deviceAuthContext.Device.Id, orgKey))
	var err error
	sp := opentracing.SpanFromContext(context.Request.Context())
	var childSp opentracing.Span = nil
	if sp != nil {
		childSp = opentracing.StartSpan("device-deregistration", opentracing.ChildOf(sp.Context()))
		defer childSp.Finish()
		defer func() {
			childSp.SetTag(string(ext.Error), err != nil)
			childSp.SetTag("org-key", orgKey)
		}()
	}
	version := context.GetInt(VersionName)
	header := deviceresponseheader.NewDeviceResponseHeader(int64(version), deviceresponseheader.HelloResponseCode, false,
		DeregisteredDeviceErrorCode, DeregisteredDeviceMessage, nil, nil, nil)
	var body string
	body, err = deviceresponseheader.NewResponseHeaderEncoder().Encode(header)
	if err != nil {
		_ = context.AbortWithError(http.StatusInternalServerError, fmt.Errorf("failed to encode response body: %w", err))
		return
	}
	context.Set("response", header)
	_, err = context.Writer.Write([]byte(body))
	if err != nil {
		_ = context.AbortWithError(http.StatusInternalServerError, fmt.Errorf("failed to write response body: %w", err))
	}
	context.AbortWithStatus(http.StatusOK)
}
