package middleware

import (
	"net/http"
	"net/http/httptest"
	"time"

	"hello-api/internal/app/parsers/devicecontext"
	"hello-api/internal/app/parsers/deviceresponseheader"

	deviceauthpb "carbonblack.com/protocols/psc/ems/server/deviceauth"
	"github.com/gin-gonic/gin"
	. "github.com/onsi/ginkgo/v2"
	. "github.com/onsi/gomega"
	"google.golang.org/protobuf/types/known/timestamppb"
)

func newDeregisteredDeviceResponseHeader(expectedVersion int64) *deviceresponseheader.DeviceResponseHeader {
	return deviceresponseheader.NewDeviceResponseHeader(expectedVersion, deviceresponseheader.HelloResponseCode, false,
		DeregisteredDeviceErrorCode, DeregisteredDeviceMessage, nil, nil, nil)
}

var _ = Describe("Device deregistration middleware", func() {
	It("returns HTTP 500 when the context header is not supplied", func() {
		recorder := httptest.NewRecorder()
		testContext, _ := gin.CreateTestContext(recorder)
		testContext.Request, _ = http.NewRequest("POST", "/hello", http.NoBody)
		DeviceDeregistrationMiddleware(testContext)
		Expect(recorder.Code).To(BeComparableTo(http.StatusInternalServerError))
	})
	It("returns HTTP 200 when the context header has a nil device", func() {
		recorder := httptest.NewRecorder()
		testContext, _ := gin.CreateTestContext(recorder)
		testContext.Request, _ = http.NewRequest("POST", "/hello", http.NoBody)
		testContext.Set(DeviceContextHeader, &devicecontext.DeviceContext{Context: deviceauthpb.Context{Device: nil}})
		DeviceDeregistrationMiddleware(testContext)
		Expect(recorder.Code).To(BeComparableTo(http.StatusOK))
		Expect(recorder.Body.String()).To(BeEquivalentTo(""))
	})
	It("returns 200 when the request is from a deregistered device", func() {
		recorder := httptest.NewRecorder()
		testContext, _ := gin.CreateTestContext(recorder)
		testContext.Request, _ = http.NewRequest("POST", "/hello", http.NoBody)
		input := &devicecontext.DeviceContext{Context: deviceauthpb.Context{Device: &deviceauthpb.Device{ChangeTimestamps: map[string]*timestamppb.Timestamp{devicecontext.DeviceDashboardDeregisterTimestampText: timestamppb.New(time.Now())}}}}
		testContext.Set(DeviceContextHeader, input)
		requestVersion := 17
		testContext.Set(VersionName, requestVersion)
		DeviceDeregistrationMiddleware(testContext)
		Expect(recorder.Code).To(BeComparableTo(http.StatusOK))
		expectedDeviceResponseHeader := newDeregisteredDeviceResponseHeader(int64(requestVersion))
		decoder := deviceresponseheader.NewResponseHeaderDecoder()
		responseBodyParsed, err := decoder.Decode(recorder.Body.String())
		Expect(err).To(BeNil())
		Expect(responseBodyParsed).To(BeEquivalentTo(expectedDeviceResponseHeader))
	})
})
