// Copyright (c) 2023 VMware Carbon Black, all rights reserved

// Package devicecontext handles the parsing of Device Context information stored in the request header
package devicecontext

import (
	"encoding/base64"
	"errors"
	"fmt"
	"strings"

	"carbonblack.com/protocols/psc/ems/server/deviceauth"

	"google.golang.org/protobuf/proto"
)

type (
	DeviceAuthResponseParser interface {
		Parse(deviceContextHeader string) (*DeviceContext, error)
	}

	deviceAuthResponseParser struct{}

	DeviceContext struct {
		deviceauth.Context
	}
)

const DeviceDashboardDeregisterTimestampText = "DEV_DASHBOARD_DEREGISTER"

func (deviceContext *DeviceContext) Validate() error {
	switch {
	case deviceContext.Device == nil:
		return fmt.Errorf("Device is nil")
	case deviceContext.DeviceGroup == nil:
		return fmt.Errorf("Group is nil")
	case deviceContext.Organization == nil:
		return fmt.Errorf("org is nil")
	default:
		return nil
	}
}

func (deviceContext *DeviceContext) IsDeregistered() bool {
	if deviceContext.Device != nil && deviceContext.Device.ChangeTimestamps[DeviceDashboardDeregisterTimestampText] != nil {
		return true
	}
	return false
}

func NewDeviceAuthResponseParser() DeviceAuthResponseParser {
	return &deviceAuthResponseParser{}
}

// Parse -- Decode the protobuf response returned as a Base64 encoded string
//   - as per 'device_auth.proto'
func (parser *deviceAuthResponseParser) Parse(deviceContextHeader string) (context *DeviceContext, err error) {
	if strings.TrimSpace(deviceContextHeader) == "" {
		return nil, errors.New("device context header was empty")
	}

	decodedHeader, err := base64.StdEncoding.DecodeString(deviceContextHeader)
	if err != nil {
		return nil, fmt.Errorf("failed to b64 decode device context header %w", err)
	}

	context = &DeviceContext{Context: deviceauth.Context{}}
	err = proto.Unmarshal(decodedHeader, &context.Context)
	if err != nil {
		return nil, fmt.Errorf("failed to unmarshal device context header %w", err)
	}

	return context, nil
}

func (deviceContext *DeviceContext) GetOrgKey() (orgKey string) {
	orgKey = "not found"
	if org := deviceContext.Organization; org != nil {
		orgKey = org.OrgKey
	}
	return
}
